package edu.berkeley.sbp;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.util.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/** a tree (or node in a tree); see jargon.txt for details */
public class Tree<T> {

    final T           head;
          Tree<T>[]   children;
    final Token.Location    location;

    public T                 head()     { return head; }
    public int               numChildren() { return children.length; }
    public Iterable<Tree<T>> children() { return new ArrayIterator(children); }
    public Tree<T>           child(int i) { return children[i]; }

    public       Token.Location    getLocation() { return location; }

    public Tree(Token.Location loc, T head)                   { this(loc, head, null); }
    public Tree(Token.Location loc, T head, Tree<T>[] children) {
        this.location = loc;
        this.head = head;
        Tree<T>[] children2 = children==null ? new Tree[0] : new Tree[children.length];
        if (children != null) System.arraycopy(children, 0, children2, 0, children.length);
        this.children = children2;
    }

    /** append Java code to <tt>sb</tt> which evaluates to this instance */
    public void toJava(StringBuffer sb) {
        sb.append("new Tree(null, ");
        sb.append(head==null ? "null" : "\"" + StringUtil.toJavaString(head+"") + "\"");
        sb.append(", new Tree[] { ");
        for(int i=0; i<children.length; i++) {
            if (children[i]==null)   sb.append("null");
            else                   children[i].toJava(sb);
            if (i<children.length-1) sb.append(",\n        ");
        }
        sb.append("})");
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        for(int i=0; i<children.length; i++) {
            String q = children[i]==null ? "null" : children[i].toString();
            if (q.length() > 0) { ret.append(q); ret.append(" "); }
        }
        String tail = ret.toString().trim();
        String h = (head!=null && !head.toString().equals("")) ? (tail.length() > 0 ? head+":" : head+"") : "";
        if (tail.length() > 0) tail = "{" + tail + "}";
        return h + tail;
    }


}
